from rest_framework import generics

# from releasemanager.models import SampleModel 
# from releasemanager.api.serializers import SampleModelSerializer


# class SampleModelListAPIView(generics.ListAPIView):
#     queryset = SampleModel.objects.filter(enabled=True)
#     serializer_class = SampleModelSerializer



# import uuid

from django.conf import settings
from django.db import models
from django.db.models import Q
from django.utils import timezone

# from django.utils.translation import gettext as _

from django.contrib.sites.models import Site


class ReleaseManager(models.Manager):
    def get_latest_release_for_package_and_user(self, package, user):
        """
        Get the latest release for a specified package within the user's release groups.
        """

        # if package is a string, get the package object
        if isinstance(package, str):
            package = Package.objects.get(package_key=package)

        current_datetime = timezone.now()

        default_id = getattr(settings, "RELEASE_MANAGER_DEFAULT_STATE", 1)

        current_site = Site.objects.get_current()

        latest_release = self.filter(
            Q(package=package),  # Filter the package
            (
                Q(release_groups__members=user)
                & Q(release_groups__sites=current_site)
                & Q(release_groups__active=True)
            )
            | (
                Q(release_date__lte=current_datetime) & Q(active=True)
            ),  # Get the latest published release, currently assumes default is ID 1 unless set in settings
        ).order_by("-release_date")

        return latest_release


class Package(models.Model):
    name = models.CharField(max_length=100, unique=True)
    package_key = models.CharField(max_length=100, unique=True)
    description = models.TextField(blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.name


def default_release_paths():
    return list([{"file_type": "css"}, {"file_type": "js"}])


class Release(models.Model):
    active = models.BooleanField(
        default=False, help_text="Is this release active for production?"
    )
    version = models.CharField(max_length=20)
    release_date = models.DateTimeField()
    package = models.ForeignKey(
        Package, on_delete=models.CASCADE, related_name="releases"
    )
    files = models.JSONField(
        blank=True,
        null=True,
        help_text="Nested type information for this release",
        default=default_release_paths,
    )

    objects = ReleaseManager()

    def __str__(self):
        return f"{self.package.name} - {self.version}"

    class Meta:
        ordering = ["-release_date"]
        unique_together = (("package", "version"),)
        # Makes sure there is only one entry of a given version per package


class ReleaseGroup(models.Model):
    """
    If a user is part of a Release Group, the newest release will be available for a given Package.

    If a user is not part of a group, they always get the latest release.
    """

    name = models.CharField(max_length=100, unique=True)
    description = models.TextField(blank=True, null=True)
    members = models.ManyToManyField(
        settings.AUTH_USER_MODEL, related_name="release_groups", blank=True
    )
    active = models.BooleanField(default=False)
    sites = models.ManyToManyField(Site, related_name="releases", blank=True)
    releases = models.ManyToManyField(
        Release, related_name="release_groups", blank=True
    )

    def __str__(self):
        return self.name


"""
# Testing the release manager

from django import template
from django.contrib.sites.models import Site
from django.db.models import Q
from django.utils import timezone
from django.contrib.auth import get_user_model

from releasemanager.models import Release, Package

User = get_user_model()

package_key = "sample_app"
current_site = Site.objects.get_current()
user = User.objects.first()

Release.objects.get_latest_release_for_package_and_user(package_key, user)
"""



from django.test import TestCase
from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site
from .models import Package, ReleaseGroup, Release
from datetime import datetime, timedelta

"""
# Data Dump
./manage.py dumpdata --indent=4 -e contenttypes.contenttype -e admin -e sessions -e auth.permission -e core.releasemanageruser > ../src/releasemanager/fixtures/release_data.json
"""


class ReleaseGroupTestCase(TestCase):
    fixtures = ["sample_user.json", "release_data.json"]

    # def test_latest_release_in_release_group(self):
    #     # Get the latest release for the package in the user's release group
    #     latest_release = Release.objects.get_latest_release_for_package_and_user(
    #         self.package, self.user
    #     )

    #     # Ensure that the latest release is the future release
    #     self.assertEqual(latest_release, self.future_release)



# Generated by Django 4.1.11 on 2023-09-08 19:43

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import releasemanager.models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Package",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100, unique=True)),
                ("package_key", models.CharField(max_length=100, unique=True)),
                ("description", models.TextField(blank=True, null=True)),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name="Release",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "active",
                    models.BooleanField(
                        default=False,
                        help_text="Is this release active for production?",
                    ),
                ),
                ("version", models.CharField(max_length=20)),
                ("release_date", models.DateTimeField()),
                (
                    "files",
                    models.JSONField(
                        blank=True,
                        default=releasemanager.models.default_release_paths,
                        help_text="Nested type information for this release",
                        null=True,
                    ),
                ),
                (
                    "package",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="releases",
                        to="releasemanager.package",
                    ),
                ),
            ],
            options={
                "ordering": ["-release_date"],
                "unique_together": {("package", "version")},
            },
        ),
        migrations.CreateModel(
            name="ReleaseGroup",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100, unique=True)),
                ("description", models.TextField(blank=True, null=True)),
                ("active", models.BooleanField(default=False)),
                (
                    "members",
                    models.ManyToManyField(
                        blank=True,
                        related_name="release_groups",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "releases",
                    models.ManyToManyField(
                        blank=True,
                        related_name="release_groups",
                        to="releasemanager.release",
                    ),
                ),
                (
                    "sites",
                    models.ManyToManyField(
                        blank=True, related_name="releases", to="sites.site"
                    ),
                ),
            ],
        ),
    ]



from django.contrib import admin

# from .models import ReleaseGroup, Release, Package
from .models import Release, Package, ReleaseGroup


# Inlines


class ReleaseInline(admin.TabularInline):
    # You can use admin.StackedInline for a different display style
    model = Release
    extra = 1  # Number of empty forms to display for adding new releases


# Admins


class ReleaseGroupAdmin(admin.ModelAdmin):
    list_display = ("name", "description")
    filter_horizontal = ("members",)


class ReleaseAdmin(admin.ModelAdmin):
    list_display = ("package", "version", "release_date", "active")
    search_fields = ("package__name", "version")


class PackageAdmin(admin.ModelAdmin):
    list_display = ("name", "description", "created_at", "updated_at")
    search_fields = ("name", "description")
    # prepopulated_fields = {"name": ("name",)}
    inlines = [ReleaseInline]


admin.site.register(ReleaseGroup, ReleaseGroupAdmin)
admin.site.register(Release, ReleaseAdmin)
admin.site.register(Package, PackageAdmin)



from django import template
from django.contrib.sites.models import Site
from django.db.models import Q
from django.utils import timezone

from releasemanager.models import Release, Package

register = template.Library()


@register.inclusion_tag("releasemanager/release_template.html")
def release_packages(package_key, user=None, release_state=None, file_types=None):
    """
    Get the latest release for a specified package within the user's release groups.
    """

    if file_types:
        file_types = file_types.split(",")
        # Convert the comma seperated string into a list

    try:
        release = Release.objects.get_latest_release_for_package_and_user(
            package_key, user
        )

        # If there is still not a release available, return an empty list
        if release.count() == 0:
            return {"files": []}

        # Optionally filter the release files by parts
        if file_types is not None:
            files = [f for f in release.first().files if f["file_type"] in file_types]
        else:
            files = release.first().files  # Use all files if 'parts' is not specified

        return {"files": files}
    except (Package.DoesNotExist, Release.DoesNotExist):
        return {"files": []}
